"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startWebDriver = startWebDriver;
exports.stopWebDriver = stopWebDriver;
exports.createSession = createSession;
exports.closeSession = closeSession;
exports.runQueue = runQueue;

var _nightwatch = require("nightwatch");

var _assertionError = _interopRequireDefault(require("assertion-error"));

var _reporter = _interopRequireDefault(require("nightwatch/lib/testsuite/reporter"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _logger = require("./logger");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let runner;
let client;

function getDefautEnvironment() {
  return 'default';
}

function getDefaultConfigFile() {
  const jsonConfigFile = './nightwatch.json';

  const jsConfigFie = _path.default.resolve('./nightwatch.conf.js');

  if (_fs.default.existsSync(jsonConfigFile)) {
    return jsonConfigFile;
  }

  if (_fs.default.existsSync(jsConfigFie)) {
    return jsConfigFie;
  }

  throw new Error(['No configuration file was found for Nightwatch in the current process folder.', '(nightwatch.json or nightwatch.conf.js).', 'For custom location please provide "configFile" option for "startWebDriver" and', '"createSession" functions.'].join(' '));
}

function createRunner({
  env = getDefautEnvironment(),
  configFile = getDefaultConfigFile()
} = {}) {
  if (!runner) {
    runner = (0, _nightwatch.CliRunner)({
      env,
      config: configFile
    });

    runner.isWebDriverManaged = function () {
      if (this.baseSettings.selenium) {
        this.baseSettings.selenium.start_process = true;
      }

      return true;
    };

    runner.setup();
  }

  return runner;
}

async function startWebDriver({
  env = getDefautEnvironment(),
  configFile = getDefaultConfigFile()
} = {}) {
  createRunner({
    env,
    configFile
  });
  const {
    port
  } = runner.test_settings.webdriver;
  await runner.startWebDriver();
  (0, _logger.log)(`WebDriver started on port ${port}`);
}

async function stopWebDriver() {
  const {
    port
  } = runner.test_settings.webdriver;
  await runner.stopWebDriver();
  (0, _logger.log)(`WebDriver stopped on port ${port}`);
}

async function createSession({
  env = getDefautEnvironment(),
  configFile = getDefaultConfigFile()
} = {}) {
  createRunner({
    env,
    configFile
  });
  const settings = runner.test_settings;
  client = (0, _nightwatch.client)(settings, new _reporter.default([], 0, {}, {}));
  await client.startSession();
  (0, _logger.log)('Session created');
  return client.api;
}

function resetQueue() {
  if (client && client.queue) {
    client.queue.reset().removeAllListeners().empty();
  }
}

async function closeSession() {
  resetQueue();

  if (client && client.queue) {
    await runQueue();
  }

  (0, _logger.log)('Session closed');
}

function handleQueueResult(err, resolve, reject) {
  if (!err) {
    resolve();
    return;
  }

  if (!(err instanceof _assertionError.default) || err.abortOnFailure) {
    resetQueue();
  }

  err.stack = [err.message, err.stack].join('\n');
  reject(err);
}

async function runQueue() {
  if (!client || !client.queue) {
    throw new Error(`Nightwatch client is not ready.
        Looks like function "createSession" did not succeed or was not called yet.`);
  }

  try {
    await new Promise((resolve, reject) => {
      client.queue.run(err => handleQueueResult(err, resolve, reject));
    });
  } catch (err) {
    throw err;
  } finally {
    resetQueue();
  }
}